/*
 *  GuiLoader/C++, a C++ binding to GuiLoader
 *  Copyright (c) 2006-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef GUILOADER_CPP_GUILOADER_CPP_IMPL_H
#define GUILOADER_CPP_GUILOADER_CPP_IMPL_H

#define GUILOADER_CPP_LIBRARY

#ifdef GUILOADER_CPP_SANDBOX
	#define G_DISABLE_DEPRECATED
	#define PANGO_DISABLE_DEPRECATED
	#define ATK_DISABLE_DEPRECATED
	#define GDK_DISABLE_DEPRECATED
	#define GDK_PIXBUF_DISABLE_DEPRECATED
	#define GTK_DISABLE_DEPRECATED
	#define GSEAL_ENABLE
	#define GLIBMM_DISABLE_DEPRECATED
	#define GIOMM_DISABLE_DEPRECATED
	#define PANGOMM_DISABLE_DEPRECATED
	#define ATKMM_DISABLE_DEPRECATED
	#define GDKMM_DISABLE_DEPRECATED
	#define GTKMM_DISABLE_DEPRECATED
#endif

#include <assert.h>
#include <guiloader.h>
#include <map>
#include <boost/format.hpp>

#include "guiloader-c++.h"

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#define GETTEXT_PACKAGE "guiloader-c++"
#include <glib/gi18n-lib.h>

#ifdef G_LOG_DOMAIN
	#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN "GuiLoader/C++"

#define CHECK(expr) ((expr) ? NoObj() : (CheckFailed(#expr, __FILE__, __LINE__), NoObj()))

namespace Gui {

struct NoObj { };

void CheckFailed(const char * expr, const char * file, int line);
void FatalError(const std::string & message);
Glib::Object * Wrap(GObject * gobj);

template <class T, class U> class TSignalConnector : public TSignal<typename T::SlotType> {

public:
	typedef T (U::*get_signal_func)();
	typedef typename T::SlotType Slot;

	TSignalConnector(get_signal_func func) {
		get_signal= func;
	}

	bool connect(Glib::Object * obj, const Slot & slot, bool after) {
		U * object= dynamic_cast<U *>(obj);
		if(object)
			(object->*get_signal)().connect(slot, after);
		return object;
	}

private:
	get_signal_func get_signal;

};

class LoaderInit {

public:
	static LoaderInit * get();

	static void enter();
	static void leave();

	Signal * findSignal(const std::string & name, GType type);

protected:
	static LoaderInit * & ptr();
	static void initGettext();

	LoaderInit();
	~LoaderInit();

	void addingSignals(GType sigclass) { this->sigclass= sigclass; }

	template <class T, class U> void addSignal(
		const std::string & name,
		T (U::*get_signal)()
	) {
		signals.insert(std::make_pair(name, std::make_pair(sigclass, new TSignalConnector<T, U>(get_signal))));
	}

private:
	int refcounter;
	GType sigclass;

	typedef std::multimap<std::string, std::pair<GType, Signal *> > Signals;
	Signals signals;

};

}

#endif
